==> dspygen-2024.3.15.tar.gz: dspygen-2024.3.15/src/dspygen/dsl/dsl_pipeline_executor.py <==
import os
from fastapi import APIRouter, HTTPException

import tempfile
from typing import Optional

import dspy
from pydantic import BaseModel

from dspygen.dsl.utils.dsl_language_model_utils import _get_language_model_instance
from dspygen.dsl.dsl_pydantic_models import PipelineDSLModel, LanguageModelConfig
from dspygen.dsl.utils.dsl_module_utils import _get_module_instance
from dspygen.dsl.utils.dsl_signature_utils import _create_signature_from_model
from dspygen.typetemp.functional import render


def execute_pipeline(file_path, initial_context=None):
    """
    Execute a pipeline from a YAML file and return the context.
    """
    pipeline = _get_pipeline(file_path)

    if initial_context:
        pipeline.context.update(initial_context)

    for step in pipeline.steps:
        _execute_step(pipeline, step)

    return pipeline.context


def _get_pipeline(file_path):
    """
    Load a PipelineDSLModel instance from a YAML file. Also, create the global signatures from the YAML.
    """
    pipeline = PipelineDSLModel.from_yaml(file_path)
    # Gather the signatures from the YAML
    pipeline.config.global_signatures = {signature.name: _create_signature_from_model(signature)
                                              for signature in pipeline.signatures}
    return pipeline


def _execute_step(pipeline, step):
    """
    Execute a step in a pipeline. Creates the LM, renders the args using Jinja2,
    runs the module, and updates the context.
    """
    if not pipeline.models:
        pipeline.models = [LanguageModelConfig(label="default", name="OpenAI", args={})]

    rendered_args = {arg: render(str(value), **pipeline.context) for arg, value in step.args.items()}

    module_inst = _get_module_instance(pipeline, rendered_args, step)

    lm_inst = _get_language_model_instance(pipeline, step)

    with dspy.context(lm=lm_inst):
        module_output = module_inst.forward(**rendered_args)

    pipeline.context[step.module] = module_output


router = APIRouter()


class PipelineRequest(BaseModel):
    yaml_content: str
    initial_context: Optional[dict] = None


@router.post("/execute_pipeline/")
async def run_pipeline(request: PipelineRequest):
    try:
        # Create a temporary file to hold the YAML content
        with tempfile.NamedTemporaryFile(delete=False, mode='w+', suffix='.yaml') as tmp:
            tmp.write(request.yaml_content)
            tmp_path = tmp.name

        context = execute_pipeline(tmp_path, request.initial_context)

        # Optionally, clean up the temporary file after execution
        os.remove(tmp_path)

        return context
    except Exception as e:
        # Ensure the temporary file is removed even if an error occurs
        if 'tmp_path' in locals():
            os.remove(tmp_path)
        raise HTTPException(status_code=500, detail=str(e))


def main():
    # context = execute_pipeline('examples/blog_pipeline.yaml')
    context = execute_pipeline('/Users/candacechatman/dev/dspygen/pipeline.yaml', {"news": "$12,500 Retainer Contract"})
    # context = execute_pipeline('examples/example_pipeline.yaml')

    print(context)


if __name__ == '__main__':
    main()

==> dspygen-2024.5.7.tar.gz: dspygen-2024.5.7/src/dspygen/dsl/dsl_pipeline_executor.py <==
import os
from fastapi import APIRouter, HTTPException

import tempfile
from typing import Optional

import dspy
from pydantic import BaseModel

from dspygen.dsl.dsl_step_module import execute_step
from dspygen.dsl.dsl_pydantic_models import PipelineDSLModel, LanguageModelConfig
from dspygen.dsl.utils.dsl_signature_utils import _create_signature_from_model
from munch import Munch


def execute_pipeline(file_path, init_ctx=None):
    """
    Execute a pipeline from a YAML file and return the context.
    """
    pipeline = _get_pipeline(file_path)

    if init_ctx:
        pipeline.context.update(init_ctx)

    for step in pipeline.steps:
        execute_step(pipeline, step)

    return Munch(pipeline.context)


def _get_pipeline(file_path):
    """
    Load a PipelineDSLModel instance from a YAML file. Also, create the global signatures from the YAML.
    """
    pipeline = PipelineDSLModel.from_yaml(file_path)
    # Gather the signatures from the YAML
    pipeline.config.global_signatures = {signature.name: _create_signature_from_model(signature)
                                              for signature in pipeline.signatures}
    return pipeline


router = APIRouter()


class PipelineRequest(BaseModel):
    yaml_content: str
    init_ctx: Optional[dict] = None


@router.post("/execute_pipeline/")
async def run_pipeline(request: PipelineRequest):
    try:
        # Create a temporary file to hold the YAML content
        with tempfile.NamedTemporaryFile(delete=False, mode='w+', suffix='.yaml') as tmp:
            tmp.write(request.yaml_content)
            tmp_path = tmp.name

        context = execute_pipeline(tmp_path, request.init_ctx)

        # Optionally, clean up the temporary file after execution
        os.remove(tmp_path)

        # Convert the context to a dictionary making sure it is JSON serializable
        context = {k: v for k, v in context.items() if isinstance(v, (str, int, float, list, dict, bool, type(None)))}

        print(f"Context:\n\n{context}\n\n")

        return context
    except Exception as e:
        # Ensure the temporary file is removed even if an error occurs
        if 'tmp_path' in locals():
            os.remove(tmp_path)
        raise HTTPException(status_code=500, detail=str(e))


def main():
    context = execute_pipeline('/Users/sac/dev/dspygen/src/dspygen/dsl/examples/example_pipeline.yaml')
    # context = execute_pipeline(str(dsl_dir('examples/text_signature_pipeline.yaml')),
    #                            {"raw_data": "id,name,job\n1,Joe,Coder"})
    # context = execute_pipeline(str(dsl_dir('examples/sql_to_nl.yaml')),
    #                            {"query": poor_query})


    print(context)

if __name__ == '__main__':
    main()
